package dk.sosi.sts.integrations.blurring;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;
import dk.sosi.sts.model.BlurredOrganization;
import dk.sosi.sts.model.CprNr;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import javax.xml.bind.JAXBException;
import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.stream.Collectors;

import static dk.sosi.sts.model.BlurredOrganization.Type.Cvr;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

public class BlurringServiceTest {
    private String responseXml;

    @Before
    public void setUp() throws IOException {
        try (InputStream is = this.getClass().getResourceAsStream("/lookupResponse.xml")) {
            responseXml = new BufferedReader(
                    new InputStreamReader(is, StandardCharsets.UTF_8))
                    .lines()
                    .collect(Collectors.joining("\n"));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    public void createRequestBody_WhenGivenInput_ReturnsRequestBody() throws JAXBException,
            ParserConfigurationException, IOException, SAXException {
        // Arrange
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        IdCardFactory idCardFactory = mock(IdCardFactory.class);
        BlurringServiceCaller blurringServiceCaller = mock(BlurringServiceCaller.class);
        BlurringService blurringService = new BlurringService(sosiFactory, idCardFactory, blurringServiceCaller);

        // Act
        Document doc = blurringService.createGetActiveBlurringsRequestBody(new CprNr("1234567890"), PatientIdClassificationType.CPR);

        // Assert
        String s = XmlUtil.node2String(doc, false, true);
        assertTrue(s.contains("<id>1234567890</id>"));
        assertTrue(s.contains("<classification>CPR</classification>"));
    }

    @Test
    public void createRequest_WhenGivenInput_ReturnsRequest() {
        // Arrange
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        IdCardFactory idCardFactory = mock(IdCardFactory.class);
        BlurringServiceCaller blurringServiceCaller = mock(BlurringServiceCaller.class);
        BlurringService blurringService = new BlurringService(sosiFactory, idCardFactory, blurringServiceCaller);

        // Return mocked Request
        Request request = mock(Request.class);
        when(sosiFactory.createNewRequest(anyBoolean(), anyString()))
                .thenReturn(request);

        // Act
        String res = blurringService.createGetActiveBlurringsRequest(new CprNr("1234567890"),
                                                                     PatientIdClassificationType.CPR,
                                                                     "dummy");

        // Assert
        assertTrue(res.contains("xml"));
    }

    @Test
    public void getActiveBlurrings_WhenCalled_ReturnsListWithSingleCVR() {
        // Arrange
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        IdCardFactory idCardFactory = mock(IdCardFactory.class);
        BlurringServiceCaller blurringServiceCaller = mock(BlurringServiceCaller.class);
        BlurringService blurringService = new BlurringService(sosiFactory, idCardFactory, blurringServiceCaller);

        // Prepare mocks
        when(blurringServiceCaller.callGetBlurredOrganisations(anyString())).thenReturn(responseXml);
        when(sosiFactory.createNewRequest(anyBoolean(), anyString()))
                .thenReturn(mock(Request.class));


        // Act
        List<BlurredOrganization> res = blurringService.getActiveBlurrings(new CprNr("1234567890"), "dummy");

        // Assert
        assertEquals(new BlurredOrganization("33257872", Cvr), res.get(0));
    }

    String expected = "TkhvHPslclPzA1jyLk+jdg==",
           response = "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
            "    <soap:Body>\n" +
            "        <SaltResponse xmlns=\"http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/\">\n" +
            "            " + expected + "</SaltResponse>\n" +
            "    </soap:Body>\n" +
            "</soap:Envelope>";

    @Test
    public void getSalt_WhenCalled_ReturnsSalt() {
        BlurringServiceCaller sc = mock(BlurringServiceCaller.class);
        when(sc.callSaltService(anyString())).thenReturn(response);
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        when(sosiFactory.createNewRequest(anyBoolean(), anyString())).thenReturn(mock(Request.class));

        BlurringService blurringService = new BlurringService(sosiFactory, mock(IdCardFactory.class), sc);
        assertEquals(expected, blurringService.getSalt("dummy"));
    }

    @Test
    public void getSalt_WhenCalled_RequestContainsFlowIdSoTheRequestCanBeTracedBetweenComponents() {
        BlurringServiceCaller sc = mock(BlurringServiceCaller.class);
        ArgumentCaptor<String> requestFlowId = ArgumentCaptor.forClass(String.class);
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        when(sosiFactory.createNewRequest(anyBoolean(), requestFlowId.capture())).thenReturn(mock(Request.class));
        String flowId = "flowId";
        BlurringService blurringService = new BlurringService(sosiFactory, mock(IdCardFactory.class), sc);

        try { blurringService.getSalt(flowId); } catch(Exception ignored) { }

        assertEquals(flowId, requestFlowId.getValue());
    }

    @Test
    public void getBlurrings_WhenCalled_RequestContainsFlowIdSoTheRequestCanBeTracedBetweenComponents() {
        BlurringServiceCaller sc = mock(BlurringServiceCaller.class);
        ArgumentCaptor<String> requestFlowId = ArgumentCaptor.forClass(String.class);
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        when(sosiFactory.createNewRequest(anyBoolean(), requestFlowId.capture())).thenReturn(mock(Request.class));
        String flowId = "flowId";
        BlurringService blurringService = new BlurringService(sosiFactory, mock(IdCardFactory.class), sc);

        try { blurringService.getActiveBlurrings(new CprNr("1234567890"), flowId); } catch(Exception ignored) { }

        assertEquals(flowId, requestFlowId.getValue());
    }
}