package dk.sosi.sts.integrations.blurring;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.SystemIDCard;
import dk.sosi.seal.model.SystemInfo;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.time.Instant;
import java.util.Date;

import static org.junit.Assert.*;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.*;
import static org.mockito.Mockito.when;

public class IdCardFactoryTest {

    @Test
    public void getIdCard_WhenCalled_ReturnsTheCreatedIdCard() {
        // Arrange
        SOSIFactory sosiFactory = Mockito.mock(SOSIFactory.class);
        IdCardFactory idCardFactory = new IdCardFactory(sosiFactory);
        idCardFactory.setCareProviderId("dummy");
        idCardFactory.setCareProviderOrg("dummy");

        SystemIDCard idCard = Mockito.mock(SystemIDCard.class);
        Mockito.when(sosiFactory.createNewSystemIDCard(Mockito.anyString(), Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any()))
                .thenReturn(idCard);

        // Act
        SystemIDCard card = idCardFactory.getIdCard();

        // Assert
        assertSame(idCard, card);
    }

    @Test
    public void getIdCard_WhenCalledWithItSystemName_UsesItSystemNameAsArgumentToSosiFactory() {
        // Arrange
        SOSIFactory sosiFactory = Mockito.mock(SOSIFactory.class);
        IdCardFactory idCardFactory = new IdCardFactory(sosiFactory);
        String itSystemName = "itSystemName", careProviderId = "id", careProviderOrg = "org";

        // Act
        idCardFactory.setItSystemName(itSystemName);
        idCardFactory.setCareProviderId(careProviderId);
        idCardFactory.setCareProviderOrg(careProviderOrg);
        idCardFactory.getIdCard();

        // Assert
        ArgumentCaptor<String> argument = ArgumentCaptor.forClass(String.class);
        ArgumentCaptor<CareProvider> cpArgument = ArgumentCaptor.forClass(CareProvider.class);
        verify(sosiFactory).createNewSystemIDCard(argument.capture(), cpArgument.capture(), Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any());
        assertEquals(itSystemName, argument.getValue());
        assertEquals(careProviderId, cpArgument.getValue().getID());
        assertEquals(careProviderOrg, cpArgument.getValue().getOrgName());
    }

    @Test
    public void getIdCard_WhenCalled_ThenExpiredIdCardShouldBeRenewed() throws ParserConfigurationException {
        SOSIFactory sosiFactory = mock(SOSIFactory.class);
        Instant now = Instant.now();
        SystemIDCard expiredIdCard = createSystemIdCard(Date.from(now.minusSeconds(300)));
        when(sosiFactory.createNewSystemIDCard(anyString(), any(), any(), anyString(), anyString(), any(), anyString()))
                .thenReturn(expiredIdCard);
        IdCardFactory idCardFactory = new IdCardFactory(sosiFactory);
        idCardFactory.setCareProviderId("id");
        idCardFactory.setCareProviderOrg("org");

        idCardFactory.getIdCard(); // Update the cache in IdCardFactory with the expired ID card from the SOSIFactory mock
        SystemIDCard validIdCard = createSystemIdCard(Date.from(now.plusSeconds(300)));
        when(sosiFactory.createNewSystemIDCard(anyString(), any(), any(), anyString(), anyString(), any(), anyString()))
                .thenReturn(validIdCard);

        assertEquals(validIdCard, idCardFactory.getIdCard());
    }

    private static SystemIDCard createSystemIdCard(Date expiryDate) throws ParserConfigurationException {
        SystemInfo systemInfo =
                new SystemInfo(new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "cvr", "navn"), "TE-STS");
        Element emptyElement =
                DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument().getDocumentElement();
        return new SystemIDCard("1.0.1",
                                emptyElement,
                                "cardId",
                                AuthenticationLevel.VOCES_TRUSTED_SYSTEM,
                                "certHash",
                                "issuer",
                                systemInfo,
                                null,
                                expiryDate,
                                "alternativeIdentifier",
                                "user",
                                "pass");
    }
}